import java.util.*;


public class Table {
	public static final int MAX_PLAYERS = 4;  //the max number of players allowed at a table
	private List<PlayerHand> playerHands;
	private String id;  //unique ID for table, to allow for multiple tables at once (tournament style)
	private int buttonPosition;  //the position (in Player list) of the dealer button. Small blind = + 1, Big blind = + 2
        private List<TablePot> pots;
        private Dealer dealer;
        private double currentBet;
	
	public Table(String id) {
            setId(id);
            buttonPosition = 0;
            playerHands = new ArrayList<PlayerHand>(MAX_PLAYERS);
            pots = new ArrayList<TablePot>();
	}
        
        public void addPotContribution(PotContribution pC, boolean notify) {
            setCurrentBet(pC);
            boolean createNew = false;  //true if we need to create a new pot for the contribution
            if(pots.size() == 0)
                createNew = true;
            else if(getActivePot().getClosed())  //if the last pot is closed (only the last pot should ever be open)
                createNew = true;
            
            if(createNew) {
                //create new TablePot
                TablePot tP = new TablePot();
                tP.addObserver(getDealer());
                tP.addContribution(pC, notify);                
                pots.add(tP);                
            } else {
                TablePot tP = getActivePot();
                tP.addContribution(pC, notify);                
            }
        }
        
        //TODO: currently only deals with one all in, cannot create multiple side pots at once
        //to be called by Dealer when a betting round is complete
        public void createSidePot(TablePot tP) {
            List<PotContribution> conts = tP.getContributions();
            double allInAmount = 0;
            for(int i = 0; i < conts.size(); i++) {
                PotContribution pC = conts.get(i);
                if(pC.getAllIn()) {
                    allInAmount = pC.getAmount();
                    break;
                }
            }
            if(allInAmount >= getCurrentBet())
                return;  //don't create a new side pot if the all in hasn't been raised;
            TablePot sidePot = new TablePot();
            for(int j = 0; j < conts.size(); j++) {
                PotContribution pC = conts.get(j);
                if(pC.getAmount() > allInAmount) {
                    PotContribution newCont = new PotContribution(pC.getPlayer(), pC.getAmount() - allInAmount, pC.getAllIn(), false);
                    pC.setAmount(allInAmount);
                    sidePot.addContribution(newCont, false);
                }
            }        
            pots.add(sidePot);
            tP.setClosed(true);
        }
        
        public TablePot getActivePot() {
            if(pots.size() > 0)
                return pots.get(pots.size() - 1);  //the active pot is always the last to be added
            else
                return null;            
        }
        
        //called when a player folds, this mucks all contributions in all pots from that player
        public void muckAllContributions(Player player) {
            List<TablePot> tPots = getPots();
            for(int i = 0; i < tPots.size(); i++) {
                TablePot tablePot = tPots.get(i);
                tablePot.muckContributions(player);
            }
        }
        
        public void clearPots() {
            pots.clear();
        }

	
	//TODO: specify where to add playerHand? ie if a player leaves, and we want to insert into the gap
	//add a playerHand to the table only if a hand for that player doesn't already exist at the table
	public void addPlayerHand(PlayerHand playerHand) {
            if(playerHands.size() <= MAX_PLAYERS) {  //check if table full
		if(playerHands.indexOf(playerHand) == -1) {
			playerHands.add(playerHand);
		}
		else {
                	System.out.println("Could not add playerHand for player ID: " + playerHand.getPlayer().getId() + ", playerHand already exists at this table for that player!");
		}
            }
            else {
		System.out.println("Table full, cannot add playerHand for player ID: " + playerHand.getPlayer().getId());
            }
	}
	
	//remove a given playerHand from table if a PlayerHand for the same Player exists at table
	public void removePlayerHand(PlayerHand playerHand) {
            if(playerHands.indexOf(playerHand) != -1) {
            	playerHands.remove(playerHand);
            }
            else {
            	System.out.println("Could not remove player hand for player ID: " + playerHand.getPlayer().getId() + ", playerHand not found for that player!");
            }
	}
	
	//move the button to the next player (for the next hand)
	public void incrementButton() {
            buttonPosition = (buttonPosition + 1) % MAX_PLAYERS;
	}
	
	public String getId() {
            return id;
	}
        
        public double getCurrentBet() {
            return currentBet;
        }
        
        public void setCurrentBet(double bet) {
            currentBet = bet;
        }
        
        private void setCurrentBet(PotContribution pC) {
            double currentBet = getCurrentBet();
            double amount = pC.getAmount();
            if(amount > getCurrentBet())
                currentBet = pC.getAmount();  //to avoid setting the current bet lower than it should be when there is a smaller all in or call
            TablePot activePot = getActivePot();
            if(activePot != null) {
                PotContribution existingPc = activePot.getPlayerContribution(pC.getPlayer());
                if(existingPc != null)
                currentBet = pC.getAmount() + existingPc.getAmount();  
            }
            setCurrentBet(currentBet);
        }
	
	public int getButtonPosition() {
		return buttonPosition;
	}
	
	public int getNumPlayerHands() {
		return getPlayerHands().size();
	}
        
        //return the number of playerHands that aren't mucked
        public int getNumActivePlayerHands() {
            List<PlayerHand> pHs = getPlayerHands();
            int count = 0;
            for(int i = 0; i < pHs.size(); i++) {
                if(!pHs.get(i).getMucked())
                    count++;
            }
            //System.out.println("Count = " + count);
            return count;
        }
	
	public List<PlayerHand> getPlayerHands() {
		return playerHands;
	}
	
	//add a card to the hand at that position relative to button, button = 0
	public void addCardToHand(Card card, int handPos) {
		if(handPos <= getNumPlayerHands()) {  //only add if valid range
			PlayerHand pH = getPlayerHand(handPos);
			pH.addCard(card);
		}
	}
	
	//get the PlayerHand at that position relative to button, button = 0
	public PlayerHand getPlayerHand(int pos) {
		int truePos = getTruePos(pos); 
		return playerHands.get(truePos);
	}
        
        private int getTruePos(int pos) {
            return (pos + getButtonPosition()) % MAX_PLAYERS;
        }
        
        public List<Player> getPlayers() {
            List<PlayerHand> hands = getPlayerHands();
            List<Player> players = new ArrayList<Player>();
            for(int i = 0; i < hands.size(); i++) {
                PlayerHand playerH = hands.get(i);
                players.add(playerH.getPlayer());
            }
            return players;
        }
	
//	public void printPlayerHands() {
//		for(int i = 0; i < playerHands.size(); i++) {
//			PlayerHand pH = playerHands.get(i);
//			Player player = pH.getPlayer();
//			System.out.print(player.getName() + ": ");
//			Set<Card> cards = pH.getCards();
//			Iterator<Card> iter = cards.iterator();
//			while(iter.hasNext()) {
//				Card card = iter.next();
//				System.out.print(card.getValue() + card.getSuit() + " ");				
//			}
//			System.out.println();
//		}
//		System.out.println();
//	}
        
        public Dealer getDealer() {
            return dealer;
        }
        
        public void setDealer(Dealer dealer) {
            this.dealer = dealer;
        }
        
        public List<TablePot> getPots() {
            return pots;
        }
        
        private void setId(String id) {
            this.id = id;
        }
}
